
`timescale 1 ns / 1 ps

	module shifter_v1_0 #
	(
                parameter integer TDATA_WIDTH                   = 32,

		// Parameters of Axi Master Bus Interface M00_AXIS
		parameter integer C_M00_AXIS_TDATA_WIDTH	= TDATA_WIDTH,

		// Parameters of Axi Slave Bus Interface S00_AXIS
		parameter integer C_S00_AXIS_TDATA_WIDTH	= TDATA_WIDTH
	)
	(
		// Users to add ports here

		// User ports ends
		// Do not modify the ports beyond this line


		// Ports of Axi Master Bus Interface M00_AXIS
		input wire  m00_axis_aclk,
		input wire  m00_axis_aresetn,
		output wire  m00_axis_tvalid,
		output wire [C_M00_AXIS_TDATA_WIDTH-1 : 0] m00_axis_tdata,
		output wire [(C_M00_AXIS_TDATA_WIDTH/8)-1 : 0] m00_axis_tstrb,
		output wire  m00_axis_tlast,
		input wire  m00_axis_tready,

		// Ports of Axi Slave Bus Interface S00_AXIS
		input wire  s00_axis_aclk,
		input wire  s00_axis_aresetn,
		output wire  s00_axis_tready,
		input wire [C_S00_AXIS_TDATA_WIDTH-1 : 0] s00_axis_tdata,
		input wire [(C_S00_AXIS_TDATA_WIDTH/8)-1 : 0] s00_axis_tstrb,
		input wire  s00_axis_tlast,
		input wire  s00_axis_tvalid
	);

        // internal buffer of one word
        reg [TDATA_WIDTH-1 : 0] buffer;
        // register of output driver
        reg [TDATA_WIDTH-1 : 0] data_out;
        // always be ready to read since our operation is single clock cycle
        assign s00_axis_tready = 1'b1;
        
        // slave driver
        always @(posedge s00_axis_aclk)
        begin
            if(!s00_axis_aresetn)
            begin
                buffer <= {(TDATA_WIDTH){1'b0}};
            end
            else
            begin
                if(s00_axis_tvalid)
                begin
                    buffer <= s00_axis_tdata;
                end
            end
        end


        // master driver
        // data can always be read from the buffer so always valid
        assign m00_axis_tvalid = 1'b1;
        assign m00_axis_tstrb  = {(TDATA_WIDTH/8){1'b1}};
        // one cycle per word so tlast always asserted
        assign m00_axis_tlast  = 1'b1;

        // assign to the output the shift operation
        assign m00_axis_tdata = data_out;

        always @(posedge m00_axis_aclk)
        begin
            if(!m00_axis_aresetn)
            begin
               data_out <= {(TDATA_WIDTH){1'b0}};
            end
            else
            begin
                data_out <= buffer >> 1;
            end
        end

	endmodule
